==> create-sdk-1.8.2-pr-1263.0.tgz: package/src/index.ts <==
#!/usr/bin/env node

import { Separator, checkbox, input, select, confirm } from "@inquirer/prompts";
import { execSync, spawnSync } from "child_process";
import pkg from "fs-extra";
import chalk from "chalk";
import { resolve } from "path";
import {
  getCommand,
  isNetlifyCLIInstalled,
  isPackageManager,
  promptEditorHelper,
} from "./utils.js";
import {
  example,
  GITIGNORE,
  createBasicIntegrationUI,
  createReadme,
  testEdgeFunction,
  BOILERPLATE_OPTIONS,
  testFunction,
  createDetails,
} from "./constants.js";

const { ensureDirSync, writeFileSync, existsSync } = pkg;

const isCLIInstalled = isNetlifyCLIInstalled();
if (!isCLIInstalled) {
  console.error(
    "The Netlify CLI must be installed in order to build, test and deploy integrations. Please install it by running:",
  );
  console.error("npm install -g netlify-cli");
  throw new Error("Netlify CLI is required");
}

const directory = await input({
  message: `Where do you want to create the integration?`,
  default: ".",
});

const destination = resolve(process.cwd(), directory);

const packageName = await input({
  message: `What is the slug of the integration? (e.g. my-integration)`,
  default: destination.split("/").pop(),
  validate: (input) => {
    // should only be lowercase letters, numbers, and dashes
    if (!/^[a-z0-9-]+$/.test(input)) {
      return "Package name can only contain lowercase letters, numbers, and dashes";
    }
    return true;
  },
});

const description = await input({
  message: `Summarize what your integration does. This will be shown on the integration's card on the Integrations page.`,
  validate: (input) => {
    if (!/^(?!.*["`])/.test(input)) {
      return "Your description can not contain double quotes or backticks.";
    }
    return true;
  },
});

const component = await select<string>({
  message: "Which boilerplate should be included?",
  choices: [
    ...Object.values(BOILERPLATE_OPTIONS),
    "None",
    "[more than one]",
  ].map((choice) => ({
    name: choice,
    value: choice,
  })),
});

let boilerplate: string[] = [];

while (!boilerplate.length) {
  boilerplate =
    component === `[more than one]`
      ? await checkbox<string>({
          message: "Which boilerplate should be included?",
          choices: Object.values(BOILERPLATE_OPTIONS).map((choice) => ({
            name: choice,
            value: choice,
          })),
        })
      : [component];
}
const hasEdgeFunctionsBoilerplate = boilerplate.includes(
  BOILERPLATE_OPTIONS.edgeFunctions,
);
const hasFunctionsBoilerplate = boilerplate.includes(
  BOILERPLATE_OPTIONS.functions,
);
const hasBuildBoilerplate = boilerplate.includes(BOILERPLATE_OPTIONS.build);
const hasConnectorBoilerplate = boilerplate.includes(
  BOILERPLATE_OPTIONS.connector,
);
const hasUIBoilerplate = boilerplate.includes(BOILERPLATE_OPTIONS.ui);

let addEdgeFunctionConfig = false;

if (hasBuildBoilerplate && !hasEdgeFunctionsBoilerplate) {
  addEdgeFunctionConfig = await select<boolean>({
    message: "Are you planning to inject edge functions into the user's site?",
    choices: [
      { name: "Yes", value: true },
      { name: "No", value: false },
    ],
  });
}

let confirmVsCodeConfigure = false;

if (addEdgeFunctionConfig || hasEdgeFunctionsBoilerplate) {
  confirmVsCodeConfigure = await confirm({
    message: "Would you like to configure VS Code to use Edge Functions?",
    default: false,
  });
}

const packageManager = await select<string>({
  message: "Which package manager do you want to use?",
  choices: ["npm", "yarn", "pnpm"].map((choice) => ({
    name: choice,
    value: choice,
  })),
});

if (!isPackageManager(packageManager)) {
  throw new Error("Invalid package manager");
}

let permissions = [];

if (hasConnectorBoilerplate) {
  permissions.push(
    "site:read",
    "site:write",
    "user:read",
    "user:write",
    "env:read",
    "env:write",
  );
} else if (
  hasBuildBoilerplate ||
  hasEdgeFunctionsBoilerplate ||
  hasFunctionsBoilerplate
) {
  permissions.push("site:read", "site:write");
} else {
  permissions.push(
    await checkbox({
      message: "Select your permission scopes",
      choices: [
        new Separator(" = Site = "),
        { name: "read", value: "site:read" },
        { name: "write", value: "site:write" },
        new Separator(" = User = "),
        { name: "read", value: "user:read" },
        { name: "write", value: "user:write" },
        new Separator(" = Environment Variables = "),
        { name: "read", value: "env:read" },
        { name: "write", value: "env:write" },
      ],
    }),
  );
}

const linkSite = await select<string>({
  message:
    "Do you want to create or link a Netlify site to your integration? This will help you test your integration and will be used to host your integration.",
  choices: ["Skip", "Link", "Create"].map((choice) => ({
    title: choice,
    value: choice.toLowerCase(),
  })),
});

if (linkSite.includes("yes")) {
  ensureDirSync(destination);

  // use spawnSync to run the command synchronously
  const linkProcess = spawnSync("netlify", ["link"], {
    stdio: "inherit",
    cwd: destination,
  });

  if (linkProcess.error) {
    throw linkProcess.error;
  }

  if (linkProcess.status !== 0) {
    throw new Error(
      `Netlify CLI exited with code ${linkProcess.status}. Please try again.`,
    );
  }

  console.log("Netlify site linked successfully.");
}
if (linkSite.includes("create")) {
  ensureDirSync(destination);

  const linkProcess = spawnSync("netlify", ["sites:create"], {
    stdio: "inherit",
    cwd: destination,
  });

  if (linkProcess.error) {
    throw linkProcess.error;
  }

  if (linkProcess.status !== 0) {
    throw new Error(
      `Netlify CLI exited with code ${linkProcess.status}. Please try again.`,
    );
  }

  console.log("Netlify site created successfully.");
}

const scripts = {
  build: "netlify-integration build -a",
  dev: "netlify-integration dev -a",
  preview: "netlify-integration preview",
};

const integrationPackageJson: Record<string, string | Record<string, string>> =
  {
    name: packageName,
    version: "0.0.1",
    main: "src/index.ts",
    type: "module",
    scripts,
    dependencies: {},
  };

if (hasConnectorBoilerplate) {
  integrationPackageJson.scripts = {
    ...scripts,
    dev: "netlify-integration dev -c",
  };
}

const integrationTsConfig = {
  compilerOptions: {
    target:
      "ES2022" /* Set the JavaScript language version for emitted JavaScript and include compatible library declarations. */,
    module: "ES2022" /* Specify what module code is generated. */,
    moduleResolution: "bundler",
    strict: true,
    rootDir: ".",
  },
  exclude: ["node_modules", "dist"],
};

ensureDirSync(destination);

if (confirmVsCodeConfigure) {
  await promptEditorHelper({ destination });
}

if (existsSync(resolve(destination, "package.json"))) {
  throw new Error(
    "There is seemingly already a package.json in this directory. Please remove it and try again.",
  );
}

writeFileSync(
  resolve(destination, "package.json"),
  JSON.stringify(integrationPackageJson, null, 2),
);
writeFileSync(
  resolve(destination, "tsconfig.json"),
  JSON.stringify(integrationTsConfig, null, 2),
);
writeFileSync(resolve(destination, ".gitignore"), GITIGNORE.trim() + "\n");

writeFileSync(
  resolve(destination, "README.md"),
  createReadme(component).trim() + "\n",
);

writeFileSync(
  resolve(destination, "details.md"),
  createDetails(packageName).trim() + "\n",
);

console.log("Adding dependencies...");

const addCommand = getCommand("add", packageManager).split(" ");
const addDeps = spawnSync(
  addCommand[0],
  [addCommand[1], "@netlify/sdk@latest", "typescript@latest"],
  {
    cwd: destination,
    stdio: "inherit",
  },
);

if (addDeps.error) {
  throw addDeps.error;
}

console.log("Initialising integration...");

execSync(
  getCommand("run", packageManager, [
    // use the local SDK bin if it exists
    process.env.LOCAL_SDK_BIN_PATH || "netlify-integration",
    "init",
    "-s",
    packageName,
    `-d "${description}"`,
    permissions.length > 0 ? `-p ${permissions.join(",")}` : "",
  ]),
  {
    cwd: destination,
    env: {
      ...process.env,
    },
  },
);

ensureDirSync(resolve(destination, "src"));
writeFileSync(
  resolve(destination, "src/index.ts"),
  "// Documentation: https://sdk.netlify.com\n" +
    example({ boilerplate, addEdgeFunctionConfig }),
);

if (hasUIBoilerplate) {
  ensureDirSync(resolve(destination, "src/ui"));
  writeFileSync(
    resolve(destination, "src/ui/index.ts"),
    createBasicIntegrationUI(packageName),
  );
}

if (addEdgeFunctionConfig || hasEdgeFunctionsBoilerplate) {
  ensureDirSync(resolve(destination, "src/edge-functions"));
  writeFileSync(
    resolve(destination, "src/edge-functions/hello-world.ts"),
    testEdgeFunction,
  );
}

if (hasFunctionsBoilerplate) {
  ensureDirSync(resolve(destination, "src/functions"));
  writeFileSync(
    resolve(destination, "src/functions/hello-world.mts"),
    testFunction,
  );
}

if (destination) {
  ensureDirSync(destination);
  // check if the directory already has a git repository
  const gitStatus = spawnSync("git", ["status"], {
    cwd: destination,
  });

  if (gitStatus.status !== 0) {
    const gitInit = spawnSync("git", ["init"], {
      cwd: destination,
    });

    const gitAdd = spawnSync("git", ["add", "."], {
      cwd: destination,
    });

    const gitCommit = spawnSync(
      "git",
      ["commit", "-m", '"Initial commit from the Netlify SDK"'],
      {
        cwd: destination,
      },
    );

    const commands = [gitInit, gitAdd, gitCommit];

    if (commands.some((cmd) => cmd.error)) {
      throw (gitInit.error || gitAdd.error || gitCommit.error)!;
    }

    if (commands.some((cmd) => cmd.status !== 0)) {
      throw new Error(
        `Git exited with code ${
          gitInit.status || gitAdd.status || gitCommit.status
        }. Please try again.`,
      );
    }
    console.log("Initialised a git repository.");
  }
}
console.log(
  chalk.cyanBright("\nYour integration is created! Time to build! 🚀"),
);

if (destination !== process.cwd()) {
  console.log(
    "\nEnter your integration directory using",
    chalk.bold.cyan(`cd ./${directory}`),
  );
}
console.log(
  "Need help? Check out the documentation at",
  chalk.bold.cyan("https://developers.netlify.com/sdk"),
);
console.log("\nDone! ✅");

==> create-sdk-2.0.0-pre.b434679.tgz: package/src/index.ts <==
#!/usr/bin/env node

import { Separator, checkbox, input, select, confirm } from "@inquirer/prompts";
import { execSync, spawnSync } from "child_process";
import pkg from "fs-extra";
import chalk from "chalk";
import { resolve } from "path";
import {
  checkConnectorBoilerplate,
  getCommand,
  isNetlifyCLIInstalled,
  isPackageManager,
  promptEditorHelper,
} from "./utils.js";
import {
  example,
  GITIGNORE,
  createReadme,
  testEdgeFunction,
  BOILERPLATE_OPTIONS,
  testFunction,
  createDetails,
} from "./constants.js";
import {
  generateUiBoilerplate,
  extensionUIDeps,
  extensionUIDevDeps,
} from "./extension-ui-boilerplate/generate-ui-boilerplate.js";
import { uiMigration } from "./ui-migration.js";

const { ensureDirSync, writeFileSync, existsSync } = pkg;

const isCLIInstalled = isNetlifyCLIInstalled();
if (!isCLIInstalled) {
  console.error(
    "The Netlify CLI must be installed in order to build, test and deploy extensions. Please install it by running:",
  );
  console.error("npm install -g netlify-cli");
  throw new Error("Netlify CLI is required");
}

const flags = process.argv.slice(2); // Get the command-line flags

if (flags.length > 0) {
  if (flags.includes("--uiMigration")) {
    await uiMigration();
    process.exit(0);
  }
}

const directory = await input({
  message: `Where do you want to create the extension?`,
  default: ".",
});

const destination = resolve(process.cwd(), directory);

const packageName = await input({
  message: `What is the slug of the extension? (e.g. my-extension)`,
  default: destination.split("/").pop(),
  validate: (input) => {
    // should only be lowercase letters, numbers, and dashes
    if (!/^[a-z0-9-]+$/.test(input)) {
      return "Package name can only contain lowercase letters, numbers, and dashes";
    }
    return true;
  },
});

const description = await input({
  message: `Summarize what your extension does. This will be shown on the extension's card in the extensions directory.`,
  validate: (input) => {
    if (!/^(?!.*["`])/.test(input)) {
      return "Your description can not contain double quotes or backticks.";
    }
    return true;
  },
});

const component = await select<string>({
  message: "Which boilerplate should be included?",
  choices: [
    ...Object.values(BOILERPLATE_OPTIONS),
    "None",
    "[more than one]",
  ].map((choice) => ({
    name: choice,
    value: choice,
  })),
});

let boilerplate: string[] = [];

while (!boilerplate.length) {
  boilerplate =
    component === `[more than one]`
      ? await checkbox<string>({
          message: "Which boilerplate should be included?",
          choices: Object.values(BOILERPLATE_OPTIONS).map((choice) => ({
            name: choice,
            value: choice,
          })),
        })
      : [component];
}
const hasEdgeFunctionsBoilerplate = boilerplate.includes(
  BOILERPLATE_OPTIONS.edgeFunctions,
);
const hasFunctionsBoilerplate = boilerplate.includes(
  BOILERPLATE_OPTIONS.functions,
);
const hasBuildBoilerplate = boilerplate.includes(BOILERPLATE_OPTIONS.build);
const hasConnectorBoilerplate = boilerplate.includes(
  BOILERPLATE_OPTIONS.connector,
);
const hasUIBoilerplate = boilerplate.includes(BOILERPLATE_OPTIONS.ui);

const connectorSupports = await checkConnectorBoilerplate(
  hasConnectorBoilerplate,
);

let addEdgeFunctionConfig = false;

if (hasBuildBoilerplate && !hasEdgeFunctionsBoilerplate) {
  addEdgeFunctionConfig = await select<boolean>({
    message: "Are you planning to inject edge functions into the user's site?",
    choices: [
      { name: "Yes", value: true },
      { name: "No", value: false },
    ],
  });
}

let confirmVsCodeConfigure = false;

if (addEdgeFunctionConfig || hasEdgeFunctionsBoilerplate) {
  confirmVsCodeConfigure = await confirm({
    message: "Would you like to configure VS Code to use Edge Functions?",
    default: false,
  });
}

const packageManager = await select<string>({
  message: "Which package manager do you want to use?",
  choices: ["npm", "yarn", "pnpm"].map((choice) => ({
    name: choice,
    value: choice,
  })),
});

if (!isPackageManager(packageManager)) {
  throw new Error("Invalid package manager");
}

let permissions = [];

if (hasConnectorBoilerplate) {
  permissions.push(
    "site:read",
    "site:write",
    "user:read",
    "user:write",
    "env:read",
    "env:write",
  );
} else if (
  hasBuildBoilerplate ||
  hasEdgeFunctionsBoilerplate ||
  hasFunctionsBoilerplate
) {
  permissions.push("site:read", "site:write");
} else {
  permissions.push(
    await checkbox({
      message: "Select your permission scopes",
      choices: [
        new Separator(" = Site = "),
        { name: "read", value: "site:read" },
        { name: "write", value: "site:write" },
        new Separator(" = User = "),
        { name: "read", value: "user:read" },
        { name: "write", value: "user:write" },
        new Separator(" = Environment Variables = "),
        { name: "read", value: "env:read" },
        { name: "write", value: "env:write" },
      ],
    }),
  );
}

const scripts = {
  build: "netlify-extension build -a",
  dev: hasUIBoilerplate
    ? "netlify-extension dev --open"
    : "netlify-extension dev -a",
};

const extensionPackageJson: Record<string, string | Record<string, string>> = {
  name: packageName,
  version: "0.0.1",
  main: "src/index.ts",
  type: "module",
  scripts,
  dependencies: {},
};

if (hasConnectorBoilerplate) {
  extensionPackageJson.scripts = {
    ...scripts,
    dev: hasUIBoilerplate
      ? "netlify-extension dev --open"
      : "netlify-extension dev -c",
  };
}

const extensionTsConfig = {
  compilerOptions: {
    target:
      "ES2022" /* Set the JavaScript language version for emitted JavaScript and include compatible library declarations. */,
    module: "ES2022" /* Specify what module code is generated. */,
    moduleResolution: "bundler",
    strict: true,
    rootDir: ".",
  },
  exclude: ["node_modules", "dist"],
};

ensureDirSync(destination);

if (confirmVsCodeConfigure) {
  await promptEditorHelper({ destination });
}

if (existsSync(resolve(destination, "package.json"))) {
  throw new Error(
    "There is seemingly already a package.json in this directory. Please remove it and try again.",
  );
}

writeFileSync(
  resolve(destination, "package.json"),
  JSON.stringify(extensionPackageJson, null, 2),
);
writeFileSync(
  resolve(destination, "tsconfig.json"),
  JSON.stringify(extensionTsConfig, null, 2),
);
writeFileSync(resolve(destination, ".gitignore"), GITIGNORE.trim() + "\n");

writeFileSync(
  resolve(destination, "README.md"),
  createReadme(component).trim() + "\n",
);

writeFileSync(
  resolve(destination, "details.md"),
  createDetails(packageName).trim() + "\n",
);

console.log("Adding dependencies...");

const deps = ["@netlify/sdk@latest"];
const devDeps = ["typescript@latest"];
if (hasUIBoilerplate) {
  deps.push(...extensionUIDeps);
  devDeps.push(...extensionUIDevDeps);
}

const addCommand = getCommand("add", packageManager).split(" ");
const addDeps = spawnSync(addCommand[0], [addCommand[1], ...deps], {
  cwd: destination,
  stdio: "inherit",
});

if (addDeps.status !== 0) {
  throw new Error(
    `Failed to add dependencies. Please try again with ${packageManager}.`,
  );
}

const addDevDeps = spawnSync(addCommand[0], [addCommand[1], ...devDeps, "-D"], {
  cwd: destination,
  stdio: "inherit",
});

if (addDevDeps.status !== 0) {
  throw new Error(
    `Failed to add dev dependencies. Please try again with ${packageManager}.`,
  );
}

console.log("Initialising extension...");

execSync(
  getCommand("run", packageManager, [
    // use the local SDK bin if it exists
    process.env.LOCAL_SDK_BIN_PATH || "netlify-extension",
    "init",
    "-s",
    packageName,
    `-d "${description}"`,
    permissions.length > 0 ? `-p ${permissions.join(",")}` : "",
  ]),
  {
    cwd: destination,
    env: {
      ...process.env,
    },
  },
);

ensureDirSync(resolve(destination, "src"));
writeFileSync(
  resolve(destination, "src/index.ts"),
  "// Documentation: https://sdk.netlify.com\n" +
    example({
      boilerplate,
      addEdgeFunctionConfig,
      connectorSupports,
    }),
);

if (hasUIBoilerplate) {
  generateUiBoilerplate(destination, boilerplate, connectorSupports);
}

if (addEdgeFunctionConfig || hasEdgeFunctionsBoilerplate) {
  ensureDirSync(resolve(destination, "src/edge-functions"));
  writeFileSync(
    resolve(destination, "src/edge-functions/hello-world.ts"),
    testEdgeFunction,
  );
}

if (hasFunctionsBoilerplate) {
  ensureDirSync(resolve(destination, "src/functions"));
  writeFileSync(
    resolve(destination, "src/functions/hello-world.mts"),
    testFunction,
  );
}

if (destination) {
  ensureDirSync(destination);
  // check if the directory already has a git repository
  const gitStatus = spawnSync("git", ["status"], {
    cwd: destination,
  });

  if (gitStatus.status !== 0) {
    const gitInit = spawnSync("git", ["init"], {
      cwd: destination,
    });

    const gitAdd = spawnSync("git", ["add", "."], {
      cwd: destination,
    });

    const gitCommit = spawnSync(
      "git",
      ["commit", "-m", '"Initial commit from the Netlify SDK"'],
      {
        cwd: destination,
      },
    );

    const commands = [gitInit, gitAdd, gitCommit];

    if (commands.some((cmd) => cmd.error)) {
      throw (gitInit.error || gitAdd.error || gitCommit.error)!;
    }

    if (commands.some((cmd) => cmd.status !== 0)) {
      throw new Error(
        `Git exited with code ${
          gitInit.status || gitAdd.status || gitCommit.status
        }. Please try again.`,
      );
    }
    console.log("Initialised a git repository.");
  }
}
console.log(chalk.cyanBright("\nYour extension is created! Time to build! 🚀"));

if (destination !== process.cwd()) {
  console.log(
    "\nEnter your extension directory using",
    chalk.bold.cyan(`cd ${directory}`),
  );
}
console.log(
  "Need help? Check out the documentation at",
  chalk.bold.cyan("https://developers.netlify.com/sdk"),
);
console.log("\nDone! ✅");
